#ifndef PSRMETADATA_HH_
#define PSRMETADATA_HH_
#include <fitsio.h>
#include <string>
#include "pvo.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"

namespace PVO
{

  namespace Meta
  {

    /** The precision for printing floats */
    const int   fltwidth=4;
    /** The precision for printing doubles */
    const int   dubwidth=6;

    std::string doubleToString(double d);
    std::string doubleToString(double d, int prec);
    std::string floatToString(float f);
    std::string intToString(int i);

    /** The information used by "pvosummary -p" */
    const std::string metadataList=
 "List of parameters used in pvosummary.\n\n\
Basic parameters written to screen:\n\
 FILENAME       The FITS file.\n\
 SRC_NAME       The SRC_NAME FITS keyword.\n\
 RAJ            Right Ascension, hh:mm:ss.ss\n\
 DECJ           Declination, dd:mm:ss.sss \n\
 BEAM FWHM      The average full-width-half-maximum of the beam\n\
                 (averaged between BMAJ and BMIN)\n\
 OBSFREQ        The central observing frequency\n\
 DATE-OBS       The UT time and date of the observation, in FITS\n\
                 format YYYY-MM-DDThh:mm:ss.ssss\n\n\
Additional parameters included only in csv file:\n\
 File Location  The full path of the FITS file.\n\
 Header Version The version of the PSRFITS definition used.\n\
 Survey         Which survey the data comes from:\n\
                  left blank if not known or not a survey file (e.g. .rf files).\n\
 Source Name    The SRC_NAME FITS keyword (printed as SRC_NAME to screen).\n\
 Telescope      Where the observations were made.\n\
 Data_type      The type of data -- usually the backend instrument used.\n\
 RAJd           Right Ascension, in decimal degrees.\n\
 DecJd          Declination, in decimal degrees.\n\
 GL             Galactic Longitude, in degrees.\n\
 GB             Galactic Latitude, in degrees.\n\
 BMAJ           The FWHM of the major axis of the beam.\n\
 BMIN           The FWHM of the minor axis of the beam.\n\
 BPA            The position angle of the beam.\n\
 MJD            The Modified Julian Date of the start of the observation.\n\
 ChanBW         The channel bandwidth (from the HISTORY table).\n\
 Nchan          Number of channels (from the HISTORY table).\n\
 Scan length    The SCANLEN keyword.\n\
 Tsamp          Time per sample [ms] (from the SUBINT table).\n\
 NBits          Number of bits per datum (from the SUBINT table).\n";
 

    /**
     * The namespace to record only relevant details about a PSRFITS
     * file, that can be used by the utility programs pvosummary and
     * pvoconesearch.
     */
    class Metadata
    {
    public:
      Metadata(){sepFlag=false; angsep=nullFloat;};
      virtual ~Metadata(){};

      void        define(FITS::PSRFits fitsfile);
      friend bool operator< (Metadata lhs, Metadata rhs){
	return (lhs.angsep < rhs.angsep);
      };

      void        setAngSep(float ra, float dec);
      bool        close(float radius);
      bool        close(float ra, float dec, float radius);

      std::string getFilename(){return filename;};
      void        setFilename(std::string s){filename=s;};
      std::string getFileloc(){return fileloc;};
      void        setFileloc(std::string s){fileloc=s;};
      std::string getSurvey(){return survey;};
      void        setSurvey(std::string s){survey=s;};
      std::string getHeaderVersion(){return headerversion;};
      void        setHeaderVersion(std::string s){headerversion=s;};
      std::string getSrcname(){return srcname;};
      void        setSrcname(std::string s){srcname=s;};
      std::string getTelescope(){return telescope;};
      void        setTelescope(std::string s){telescope=s;};
      std::string getDataType(){return data_type;};
      void        setDataType(std::string s){data_type=s;};
      std::string getRaS(){return raj;};
      void        setRaS(std::string s){raj=s;};
      std::string getDecS(){return decj;};
      void        setDecS(std::string s){decj=s;};
      float       getRaD(){return rajd;};
      void        setRaD(float f){rajd=f;};
      float       getDecD(){return decjd;};
      void        setDecD(float f){decjd=f;};
      std::string getDate(){return date;};
      void        setDate(std::string s){date=s;};
      double      getGL(){return gl;};
      void        setGL(double d){gl=d;};
      double      getGB(){return gb;};
      void        setGB(double d){gb=d;};
      float       getBeamsize(){return beamsize;};
      void        setBeamsize(float f){beamsize=f;};
      float       getBmaj(){return bmaj;};
      void        setBmaj(float f){bmaj=f;};
      float       getBmin(){return bmin;};
      void        setBmin(float f){bmin=f;};
      float       getBpa(){return bpa;};
      void        setBpa(float f){bpa=f;};
      double      getMJD(){return mjd;};
      void        setMJD(double d){mjd=d;};
      float       getObsfreq(){return obsfreq;};
      void        setObsfreq(float f){obsfreq=f;};
      float       getTsamp(){return tsamp;};
      void        setTsamp(float f){tsamp=f;};
      int         getNbits(){return nbits;};
      void        setNbits(int i){nbits=i;};
      float       getScanlen(){return scanlen;};
      void        setScanlen(float f){scanlen=f;};
      int         getNchan(){return nchan;};
      void        setNchan(int i){nchan=i;};
      float       getChanbw(){return chanbw;};
      void        setChanbw(float f){chanbw=f;};
      float       getAngSep(){if(sepFlag) return angsep; else return -1.;};
      void        setAngSep(float f){angsep=f; sepFlag=true;};
      bool        showSep(){return sepFlag;};

      std::string getBeamsizeS();
      std::string getObsfreqS();
      std::string getTbinS();
      std::string getAngSepS();

    private:
      std::string filename;
      std::string fileloc;
      std::string headerversion;
      std::string survey;
      std::string srcname;
      std::string telescope;
      std::string data_type;
      std::string raj;
      std::string decj;
      float       rajd;
      float       decjd;
      double      gl;
      double      gb;
      std::string date;
      float       beamsize;
      float       bmaj;
      float       bmin;
      float       bpa;
      double      mjd;
      float       obsfreq;
      float       tsamp;
      float       scanlen;
      int         nbits;
      int         nchan;
      float       chanbw;
      float       angsep;
      bool        sepFlag;
    };

    /** Print summary of a set of metadata to a stream */
    void printMetadataShort(std::ostream &theStream, Metadata data);
    /** Print summary of a list of metadata sets to a stream */
    void printMetadataShort(std::ostream &theStream, std::vector<Metadata> datalist);
    /** Print an entire set of metadata to a stream */
    void printMetadataFull(std::string filename, Metadata data);
    /** Print the full set of a list of metadata sets to a stream */
    void printMetadataFull(std::string filename, std::vector<Metadata> datalist);

  }
}

#endif //PSRSUMMARY_HH_
